package be.iminds.ilabt.jfed.experimenter_gui.gts_editor;

import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.actions.GTSEditorActionsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.views.GTSViewsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.ui.GtsEditorTabHost;
import be.iminds.ilabt.jfed.gts.model.impl.FXGtsType;

import javax.inject.Inject;
import javax.inject.Singleton;

/**
 * User: twalcari
 * Date: 12/8/2014
 * Time: 15:58
 */
@Singleton
public class GtsEditorTabFactory {

    private final GtsEditorTabHost gtsEditorTabHost;

    private final GTSViewsFactory gtsViewsFactory;
    private final GTSEditorActionsFactory gtsEditorActionsFactory;


    @Inject
    public GtsEditorTabFactory(GtsEditorTabHost gtsEditorTabHost, GTSViewsFactory gtsViewsFactory,
                               GTSEditorActionsFactory gtsEditorActionsFactory) {
        this.gtsEditorTabHost = gtsEditorTabHost;
        this.gtsViewsFactory = gtsViewsFactory;
        this.gtsEditorActionsFactory = gtsEditorActionsFactory;
    }


    public GtsTopTypeEditorTab createGtsTopTypeEditorTab(EditableGtsDsl editableGtsDsl) {
        if (editableGtsDsl.isEmpty()) {
            editableGtsDsl.setModel(new FXGtsType(editableGtsDsl.getName()));
        }

        return new GtsTopTypeEditorTab(editableGtsDsl, gtsViewsFactory, gtsEditorActionsFactory);
    }

    public GtsChildTypeEditorTab createGtsChildTypeEditorTab(FXGtsType type, GTSModelEditor parentModel) {
        return new GtsChildTypeEditorTab(type, parentModel, gtsViewsFactory);
    }

    /**
     * @return A new unique experiment definition name
     */
    private String getNextNewGtsDslName() {
        String baseName = "UntitledGts";

        if (!experimentDefinitionNameExists(baseName))
            return baseName;

        int counter = 1;
        String name = baseName + counter;
        while (experimentDefinitionNameExists(name)) {
            counter++;
            name = baseName + counter;
        }
        return name;
    }


    /**
     * Tests whether a given experiment definition is already open in the GUI
     */
    private boolean experimentDefinitionNameExists(String name) {
        return gtsEditorTabHost.getGtsTopTypeEditorTabs()
                .stream()
                .map(GtsTopTypeEditorTab::getGtsTopTypeEditor)
                .map(GtsTopTypeEditor::getEditableGtsDsl)
                .map(EditableGtsDsl::getName)
                .anyMatch(name::equals);
    }

}
