package be.iminds.ilabt.jfed.experimenter_gui.slice.espec.view;

import be.iminds.ilabt.jfed.espec.model.AnsiblePlaybookSpec;
import be.iminds.ilabt.jfed.espec.model.ExecuteSpec;
import be.iminds.ilabt.jfed.experiment.ExperimentController;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecLogItem;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecLogModel;
import be.iminds.ilabt.jfed.experimenter_gui.slice.espec.model.ESpecStateItem;
import be.iminds.ilabt.jfed.experimenter_gui.ui.status.TaskStatusIndicator;
import javafx.beans.binding.ObjectBinding;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.StringProperty;
import javafx.collections.ListChangeListener;
import javafx.event.Event;
import javafx.event.EventHandler;
import javafx.scene.Cursor;
import javafx.scene.Node;
import javafx.scene.control.*;
import javafx.scene.input.MouseButton;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.HBox;

import javax.annotation.Nonnull;


public class ESpecLogList extends ListView<ESpecLogItem> {
    private final ExperimentController experimentController;

    public ESpecLogList(ExperimentController experimentController, ESpecLogModel eSpecLogModel) {
        this.experimentController = experimentController;
        setPrefHeight(100);
        setCellFactory(progressItemListView -> new ESpecViewItem());

        setItems(eSpecLogModel.getESpecLogItems());

        //always show the last progressitem that was added
        ListChangeListener<ESpecLogItem> showLastItem = c -> {
            while (c.next()) {
                if (c.wasAdded()) {
                    scrollTo(getItems().size() - 1);
                }
            }
        };
        getItems().addListener(showLastItem);

        eSpecLogModel.selectedESpecStateItemProperty().addListener((o, oldVal, newVal) -> {
            if (oldVal != newVal) {
                getItems().removeListener(showLastItem);
                if (newVal != null) {
                    setItems(newVal.getLogItems());
                } else {
                    setItems(eSpecLogModel.getESpecLogItems());
                }
                getItems().addListener(showLastItem);
                scrollTo(getItems().size() - 1);
            }
        });
    }

    private class ESpecViewItem extends ListCell<ESpecLogItem> {

        private final HBox hbox;

        private final TaskStatusIndicator taskStatusIndicator;
        private final Label textLabel;
        private final Label statusLabel;

        private final Label logDetailsLabel;
        private final ProgressBar uploadProgressBar;

        private final ContextMenu contextMenu;
        private final MenuItem rerunSingle;
        private final MenuItem rerunFrom;
        private final MenuItem rerunAll;
//        private final MenuItem rerunFromNext;
        private final MenuItem rerunBefore;

        public ESpecViewItem() {
            contextMenu = new ContextMenu();
            //TODO: improve this bad UX
            rerunSingle = new MenuItem("Re-run this step, on all involved nodes");
            rerunFrom = new MenuItem("Re-run from this step, on all involved nodes");
            rerunAll = new MenuItem("Re-run all steps on all nodes");
//            rerunFromNext = new MenuItem("Re-run from next step, on all involved nodes");
            rerunBefore = new MenuItem("Re-run up to and including this step, on all involved nodes");

            rerunAll.setOnAction((ae) -> experimentController.rerunESpec(null, null));

            contextMenu.getItems().setAll(rerunSingle, rerunFrom, rerunBefore, rerunAll);

            hbox = new HBox();
            hbox.setSpacing(5);

            // add taskStatusIndicator as graphic
            taskStatusIndicator = new TaskStatusIndicator();
            taskStatusIndicator.setPrefHeight(16);
            taskStatusIndicator.setPrefWidth(16);

            textLabel = new Label();
            statusLabel = new Label();
            statusLabel.setStyle("-fx-text-fill: darkgrey;");

            logDetailsLabel = new Label("");
            logDetailsLabel.setStyle("-fx-text-fill: darkgrey; -fx-font-scale: 0.75;");

            uploadProgressBar = new ProgressBar();

            hbox.getChildren().addAll(taskStatusIndicator, textLabel, statusLabel, logDetailsLabel, uploadProgressBar);

            logDetailsLabel.setVisible(false);
            uploadProgressBar.setVisible(false);
            logDetailsLabel.managedProperty().bind(logDetailsLabel.visibleProperty());
            uploadProgressBar.managedProperty().bind(uploadProgressBar.visibleProperty());

            //setContextMenu(contextMenu);

            setGraphic(hbox);
        }

        private EventHandler<Event> createExecuteStepOnMouseClickedHandler(final ESpecLogItem item) {
            return (e) -> {
                MouseEvent mouseEvent = (MouseEvent) e;
                if (e.getEventType() != MouseEvent.MOUSE_CLICKED)
                    return;
                if (mouseEvent.getButton() == MouseButton.PRIMARY) {
                    if (item.getLimitedLiveLog() != null) {
                        Dialog logDialog = LimitedLiveLogPanel.createAsDialog(
                                e.getSource() instanceof Node ? (Node) e.getSource() : null,
                                item.getLimitedLiveLog());
                        logDialog.show();
                    }
                } else if (mouseEvent.getButton() == MouseButton.SECONDARY) {
                    rerunSingle.setOnAction((ae) -> experimentController.rerunESpec(item.getEspecStep(), item.getEspecStep()));
                    rerunFrom.setOnAction((ae) -> experimentController.rerunESpec(item.getEspecStep(), null));
                    rerunAll.setOnAction((ae) -> experimentController.rerunESpec(null, null));
//                    rerunFromNext.setOnAction((ae) -> experimentController.rerunESpec(item.getEspecStep()todo, null));
                    rerunBefore.setOnAction((ae) -> experimentController.rerunESpec(null, item.getEspecStep()));

                    contextMenu.getItems().setAll(rerunSingle, rerunFrom, rerunBefore, rerunAll);
                    contextMenu.show(this, mouseEvent.getScreenX(), mouseEvent.getScreenY());
                }
            };

            //TODO
            //for left mouse button, call log view panel, if there is one (see args)
            //for right mouse, call rerunESpec with  button dropdown menu
        }

        private EventHandler<Event> createNonExecuteStepOnMouseClickedHandler(final ESpecLogItem item) {
            return (e) -> {
                MouseEvent mouseEvent = (MouseEvent) e;
                if (e.getEventType() != MouseEvent.MOUSE_CLICKED)
                    return;
                if (mouseEvent.getButton() == MouseButton.SECONDARY) {
                    contextMenu.getItems().setAll(rerunAll);
                    contextMenu.show(this, mouseEvent.getScreenX(), mouseEvent.getScreenY());
                }
            };

            //TODO
            //for left mouse button, call log view panel, if there is one (see args)
            //for right mouse, call rerunESpec with  button dropdown menu
        }

        @Override
        protected void updateItem(final ESpecLogItem item, boolean empty) {
            super.updateItem(item, empty);

            logDetailsLabel.visibleProperty().unbind();
            uploadProgressBar.visibleProperty().unbind();

            if (!empty) {
                textLabel.tooltipProperty().bind(new TooltipBinding(item.eSpecStateItemProperty()));
                textLabel.textProperty().bind(item.textProperty());
                statusLabel.textProperty().bind(item.statusTextProperty());
                taskStatusIndicator.statusProperty().bind(item.statusProperty());
                taskStatusIndicator.setVisible(true);

                logDetailsLabel.visibleProperty().bind(item.logByteSizeProperty().greaterThan(0));
                logDetailsLabel.textProperty().bind(item.logByteSizeProperty().asString("(log size = %d bytes)"));

                uploadProgressBar.progressProperty().bind(item.progressProperty());
                uploadProgressBar.visibleProperty().bind(item.progressProperty().greaterThan(0.0));

                setOnMouseEntered(mouseEvent -> {
                    if (item.getEspecStep() instanceof ExecuteSpec ||
                            item.getEspecStep() instanceof AnsiblePlaybookSpec) {
                        //these have logs and can be restarted
                        setOnMouseClicked(createExecuteStepOnMouseClickedHandler(item));
                        setCursor(Cursor.HAND);
                    } else {
                        setOnMouseClicked(createNonExecuteStepOnMouseClickedHandler(item));
//                        setOnMouseClicked(null);
                        setCursor(null);
                    }
                });

            } else {
                textLabel.textProperty().unbind();
                textLabel.setText("");
                statusLabel.textProperty().unbind();
                statusLabel.setText("");
                taskStatusIndicator.setVisible(false);
                setOnMouseClicked(null);
                setCursor(null);
                logDetailsLabel.setVisible(false);
                uploadProgressBar.setVisible(false);
            }
        }
    }





    private static class TooltipStringBinding extends ObjectBinding<Tooltip> {
        @Nonnull private final StringProperty text;

        public TooltipStringBinding(@Nonnull StringProperty text) {
            bind(text);
            this.text = text;
        }

        @Override
        protected Tooltip computeValue() {
            return new Tooltip(text.get());
        }
    }
    private static class TooltipBinding extends ObjectBinding<Tooltip> {
        @Nonnull private final ObjectProperty<ESpecStateItem> item;

        public TooltipBinding(@Nonnull ObjectProperty<ESpecStateItem> item) {
            bind(item);
            this.item = item;
        }

        @Override
        protected Tooltip computeValue() {
            if (item.get() != null)
                return new Tooltip("Occured during state "+item.get().getHumanName());
            else
                return new Tooltip("Occured during null state ");
        }
    }
}
